@@ -19,19 +19,13 @@
 
 /**
  * SECTION:element-alsamixer
- * @short_description: control properties of an audio device
  * @see_also: alsasink, alsasrc
  *
- * <refsect2>
- * <para>
  * This element controls various aspects such as the volume and balance
  * of an audio device using the ALSA api.
- * </para>
- * <para>
+ *
  * The application should query and use the interfaces provided by this 
  * element to control the device.
- * </para>
- * </refsect2>
  *
  * Last reviewed on 2006-03-01 (0.10.4)
  */
@@ -41,6 +35,7 @@
 #endif
 
 #include "gstalsamixer.h"
+#include <errno.h>
 
 static void gst_alsa_mixer_update_option (GstAlsaMixer * mixer,
     GstAlsaMixerOptions * alsa_opts);
@@ -135,11 +130,14 @@ gst_alsa_mixer_find_master_mixer (GstAlsaMixer * mixer, snd_mixer_t * handle)
 
   count = snd_mixer_get_count (handle);
 
+  g_static_rec_mutex_lock (mixer->rec_mutex);
+
   /* Check if we have a playback mixer labelled as 'Master' */
   element = snd_mixer_first_elem (handle);
   for (i = 0; i < count; i++) {
     if (snd_mixer_selem_has_playback_volume (element) &&
         strcmp (snd_mixer_selem_get_name (element), "Master") == 0) {
+      g_static_rec_mutex_unlock (mixer->rec_mutex);
       return element;
     }
     element = snd_mixer_elem_next (element);
@@ -150,6 +148,7 @@ gst_alsa_mixer_find_master_mixer (GstAlsaMixer * mixer, snd_mixer_t * handle)
   for (i = 0; i < count; i++) {
     if (snd_mixer_selem_has_playback_volume (element) &&
         strcmp (snd_mixer_selem_get_name (element), "Front") == 0) {
+      g_static_rec_mutex_unlock (mixer->rec_mutex);
       return element;
     }
     element = snd_mixer_elem_next (element);
@@ -160,16 +159,42 @@ gst_alsa_mixer_find_master_mixer (GstAlsaMixer * mixer, snd_mixer_t * handle)
   for (i = 0; i < count; i++) {
     if (snd_mixer_selem_has_playback_volume (element) &&
         strcmp (snd_mixer_selem_get_name (element), "PCM") == 0) {
+      g_static_rec_mutex_unlock (mixer->rec_mutex);
       return element;
     }
     element = snd_mixer_elem_next (element);
   }
 
-  /* If not, check if we have a playback mixer with both volume and switch */
+  /* If not, check if we have a playback mixer labelled as 'Speaker' */
+  element = snd_mixer_first_elem (handle);
+  for (i = 0; i < count; i++) {
+    if (snd_mixer_selem_has_playback_volume (element) &&
+        strcmp (snd_mixer_selem_get_name (element), "Speaker") == 0) {
+      g_static_rec_mutex_unlock (mixer->rec_mutex);
+      return element;
+    }
+    element = snd_mixer_elem_next (element);
+  }
+
+  /* If not, check if we have a playback mixer with both volume and switch that
+   * is not mono */
+  element = snd_mixer_first_elem (handle);
+  for (i = 0; i < count; i++) {
+    if (snd_mixer_selem_has_playback_volume (element) &&
+        snd_mixer_selem_has_playback_switch (element) &&
+        !snd_mixer_selem_is_playback_mono (element)) {
+      g_static_rec_mutex_unlock (mixer->rec_mutex);
+      return element;
+    }
+    element = snd_mixer_elem_next (element);
+  }
+
+  /* If not, check if we have any playback mixer with both volume and switch */
   element = snd_mixer_first_elem (handle);
   for (i = 0; i < count; i++) {
     if (snd_mixer_selem_has_playback_volume (element) &&
         snd_mixer_selem_has_playback_switch (element)) {
+      g_static_rec_mutex_unlock (mixer->rec_mutex);
       return element;
     }
     element = snd_mixer_elem_next (element);
@@ -179,11 +204,13 @@ gst_alsa_mixer_find_master_mixer (GstAlsaMixer * mixer, snd_mixer_t * handle)
   element = snd_mixer_first_elem (handle);
   for (i = 0; i < count; i++) {
     if (snd_mixer_selem_has_playback_volume (element)) {
+      g_static_rec_mutex_unlock (mixer->rec_mutex);
       return element;
     }
     element = snd_mixer_elem_next (element);
   }
 
+  g_static_rec_mutex_unlock (mixer->rec_mutex);
   /* Looks like we're out of luck ... */
   return NULL;
 }
@@ -392,6 +419,7 @@ task_monitor_alsa (gpointer data)
   unsigned int nfds, rnfds;
   unsigned short revents;
   GstAlsaMixer *mixer = (GstAlsaMixer *) data;
+  gint ret;
 
   g_static_rec_mutex_lock (mixer->rec_mutex);
 
@@ -399,6 +427,7 @@ task_monitor_alsa (gpointer data)
   if (nfds <= 0) {
     GST_ERROR ("snd_mixer_poll_descriptors_count <= 0: %d", nfds);
     /* FIXME: sleep ? stop monitoring ? */
+    g_static_rec_mutex_unlock (mixer->rec_mutex);
     return;
   }
 
@@ -406,6 +435,14 @@ task_monitor_alsa (gpointer data)
   rnfds = snd_mixer_poll_descriptors (mixer->handle, pfds, nfds);
   g_assert (rnfds <= nfds);
 
+  if (rnfds < 0) {
+    GST_ELEMENT_ERROR (mixer, RESOURCE, READ, (NULL), ("alsa error: %s",
+            snd_strerror (rnfds)));
+    gst_task_pause (mixer->task);
+    g_static_rec_mutex_unlock (mixer->rec_mutex);
+    return;
+  }
+
   pfds[rnfds].fd = mixer->pfd[0];
   pfds[rnfds].events = POLLIN | POLLPRI | POLLHUP | POLLERR;
   pfds[rnfds].revents = 0;
@@ -413,14 +450,28 @@ task_monitor_alsa (gpointer data)
   g_static_rec_mutex_unlock (mixer->rec_mutex);
 
   GST_LOG ("task loop");
-  poll (pfds, rnfds + 1, -1);
+  ret = poll (pfds, rnfds + 1, -1);
+
+  if (ret < 0) {
+    GST_ELEMENT_ERROR (mixer, RESOURCE, READ, (NULL), GST_ERROR_SYSTEM);
+    gst_task_pause (mixer->task);
+    return;
+  }
 
   g_static_rec_mutex_lock (mixer->rec_mutex);
 
-  snd_mixer_poll_descriptors_revents (mixer->handle, pfds, nfds, &revents);
-  if (revents & POLLIN || revents & POLLPRI) {
+  ret =
+      snd_mixer_poll_descriptors_revents (mixer->handle, pfds, nfds, &revents);
+  if (ret < 0) {
+    GST_ELEMENT_ERROR (mixer, RESOURCE, READ, (NULL), ("alsa error: %s",
+            snd_strerror (ret)));
+    gst_task_pause (mixer->task);
+  } else if (revents & (POLLIN | POLLPRI)) {
     GST_DEBUG ("Handling events");
     snd_mixer_handle_events (mixer->handle);
+  } else if (revents & (POLLERR | POLLNVAL | POLLHUP)) {
+    GST_ELEMENT_ERROR (mixer, RESOURCE, READ, (NULL), (NULL));
+    gst_task_pause (mixer->task);
   }
 
   g_static_rec_mutex_unlock (mixer->rec_mutex);
@@ -555,6 +606,8 @@ gst_alsa_mixer_get_volume (GstAlsaMixer * mixer, GstMixerTrack * track,
 
   g_return_if_fail (mixer->handle != NULL);
 
+  g_static_rec_mutex_lock (mixer->rec_mutex);
+
   gst_alsa_mixer_track_update (alsa_track);
 
   if (track->flags & GST_MIXER_TRACK_OUTPUT) {  /* return playback volume */
@@ -589,6 +642,7 @@ gst_alsa_mixer_get_volume (GstAlsaMixer * mixer, GstMixerTrack * track,
         volumes[i] = alsa_track->volumes[i];
     }
   }
+  g_static_rec_mutex_unlock (mixer->rec_mutex);
 }
 
 static gboolean
@@ -616,6 +670,8 @@ gst_alsa_mixer_set_volume (GstAlsaMixer * mixer, GstMixerTrack * track,
 
   g_return_if_fail (mixer->handle != NULL);
 
+  g_static_rec_mutex_lock (mixer->rec_mutex);
+
   gst_alsa_mixer_track_update (alsa_track);
 
   if (track->flags & GST_MIXER_TRACK_OUTPUT) {
@@ -662,6 +718,7 @@ gst_alsa_mixer_set_volume (GstAlsaMixer * mixer, GstMixerTrack * track,
         alsa_track->volumes[i] = volumes[i];
     }
   }
+  g_static_rec_mutex_unlock (mixer->rec_mutex);
 }
 
 void
@@ -672,11 +729,14 @@ gst_alsa_mixer_set_mute (GstAlsaMixer * mixer, GstMixerTrack * track,
 
   g_return_if_fail (mixer->handle != NULL);
 
+  g_static_rec_mutex_lock (mixer->rec_mutex);
+
   gst_alsa_mixer_track_update (alsa_track);
 
-  if (!!(mute) == !!(track->flags & GST_MIXER_TRACK_MUTE))
+  if (!!(mute) == !!(track->flags & GST_MIXER_TRACK_MUTE)) {
+    g_static_rec_mutex_unlock (mixer->rec_mutex);
     return;
-
+  }
   if (mute) {
     track->flags |= GST_MIXER_TRACK_MUTE;
 
@@ -705,11 +765,12 @@ gst_alsa_mixer_set_mute (GstAlsaMixer * mixer, GstMixerTrack * track,
 
     for (i = 0; i < ((GstMixerTrack *) ctrl_track)->num_channels; i++) {
       long vol =
-          mute ? ((GstMixerTrack *) ctrl_track)->min_volume : ctrl_track->
-          volumes[i];
+          mute ? ((GstMixerTrack *) ctrl_track)->
+          min_volume : ctrl_track->volumes[i];
       snd_mixer_selem_set_playback_volume (ctrl_track->element, i, vol);
     }
   }
+  g_static_rec_mutex_unlock (mixer->rec_mutex);
 }
 
 void
@@ -720,10 +781,14 @@ gst_alsa_mixer_set_record (GstAlsaMixer * mixer,
 
   g_return_if_fail (mixer->handle != NULL);
 
+  g_static_rec_mutex_lock (mixer->rec_mutex);
+
   gst_alsa_mixer_track_update (alsa_track);
 
-  if (!!(record) == !!(track->flags & GST_MIXER_TRACK_RECORD))
+  if (!!(record) == !!(track->flags & GST_MIXER_TRACK_RECORD)) {
+    g_static_rec_mutex_unlock (mixer->rec_mutex);
     return;
+  }
 
   if (record) {
     track->flags |= GST_MIXER_TRACK_RECORD;
@@ -761,6 +826,7 @@ gst_alsa_mixer_set_record (GstAlsaMixer * mixer,
       snd_mixer_selem_set_capture_volume (alsa_track->element, i, vol);
     }
   }
+  g_static_rec_mutex_unlock (mixer->rec_mutex);
 }
 
 void
@@ -782,7 +848,9 @@ gst_alsa_mixer_set_option (GstAlsaMixer * mixer,
   if (idx == -1)
     return;
 
+  g_static_rec_mutex_lock (mixer->rec_mutex);
   snd_mixer_selem_set_enum_item (alsa_opts->element, 0, idx);
+  g_static_rec_mutex_unlock (mixer->rec_mutex);
 }
 
 const gchar *
@@ -793,8 +861,9 @@ gst_alsa_mixer_get_option (GstAlsaMixer * mixer, GstMixerOptions * opts)
   GstAlsaMixerOptions *alsa_opts = GST_ALSA_MIXER_OPTIONS (opts);
 
   g_return_val_if_fail (mixer->handle != NULL, NULL);
-
+  g_static_rec_mutex_lock (mixer->rec_mutex);
   ret = snd_mixer_selem_get_enum_item (alsa_opts->element, 0, &idx);
+  g_static_rec_mutex_unlock (mixer->rec_mutex);
   if (ret == 0)
     return g_list_nth_data (opts->values, idx);
   else
@@ -821,8 +890,9 @@ gst_alsa_mixer_update_option (GstAlsaMixer * mixer,
     GST_WARNING ("Cannot send update notifications, no GstMixer * given");
     return;
   }
-
+  g_static_rec_mutex_lock (mixer->rec_mutex);
   ret = snd_mixer_selem_get_enum_item (alsa_opts->element, 0, &idx);
+  g_static_rec_mutex_unlock (mixer->rec_mutex);
   if (ret == 0) {
     option = g_list_nth_data (GST_MIXER_OPTIONS (alsa_opts)->values, idx);
     gst_mixer_option_changed (mixer->interface, GST_MIXER_OPTIONS (alsa_opts),
