@@ -47,7 +47,6 @@ struct _GstVisual
   /* pads */
   GstPad *sinkpad;
   GstPad *srcpad;
-  GstClockTime next_ts;
   GstSegment segment;
 
   /* libvisual stuff */
@@ -181,14 +180,7 @@ gst_visual_class_init (gpointer g_class, gpointer class_data)
   if (class_data == NULL) {
     parent_class = g_type_class_peek_parent (g_class);
   } else {
-    GstElementDetails details = {
-      NULL,
-      "Visualization",
-      klass->plugin->info->about,
-      "Benjamin Otte <otte@gnome.org>"
-    };
-
-    details.longname = g_strdup_printf ("libvisual %s plugin v.%s",
+    char *longname = g_strdup_printf ("libvisual %s plugin v.%s",
         klass->plugin->info->name, klass->plugin->info->version);
 
     /* FIXME: improve to only register what plugin supports? */
@@ -196,8 +188,11 @@ gst_visual_class_init (gpointer g_class, gpointer class_data)
         gst_static_pad_template_get (&src_template));
     gst_element_class_add_pad_template (element,
         gst_static_pad_template_get (&sink_template));
-    gst_element_class_set_details (element, &details);
-    g_free (details.longname);
+    gst_element_class_set_details_simple (element,
+        longname, "Visualization",
+        klass->plugin->info->about, "Benjamin Otte <otte@gnome.org>");
+
+    g_free (longname);
   }
 
   object->dispose = gst_visual_dispose;
@@ -256,7 +251,6 @@ gst_visual_dispose (GObject * object)
 static void
 gst_visual_reset (GstVisual * visual)
 {
-  visual->next_ts = -1;
   gst_adapter_clear (visual->adapter);
   gst_segment_init (&visual->segment, GST_FORMAT_UNDEFINED);
 
@@ -576,13 +570,8 @@ gst_visual_chain (GstPad * pad, GstBuffer * buffer)
   /* resync on DISCONT */
   if (GST_BUFFER_FLAG_IS_SET (buffer, GST_BUFFER_FLAG_DISCONT)) {
     gst_adapter_clear (visual->adapter);
-    visual->next_ts = -1;
   }
 
-  /* Match timestamps from the incoming audio */
-  if (GST_BUFFER_TIMESTAMP (buffer) != GST_CLOCK_TIME_NONE)
-    visual->next_ts = GST_BUFFER_TIMESTAMP (buffer);
-
   GST_DEBUG_OBJECT (visual,
       "Input buffer has %d samples, time=%" G_GUINT64_FORMAT,
       GST_BUFFER_SIZE (buffer) / visual->bps, GST_BUFFER_TIMESTAMP (buffer));
@@ -592,6 +581,7 @@ gst_visual_chain (GstPad * pad, GstBuffer * buffer)
   while (TRUE) {
     gboolean need_skip;
     const guint16 *data;
+    guint64 dist, timestamp;
 
     GST_DEBUG_OBJECT (visual, "processing buffer");
 
@@ -606,12 +596,20 @@ gst_visual_chain (GstPad * pad, GstBuffer * buffer)
     if (avail < visual->spf * visual->bps)
       break;
 
-    if (visual->next_ts != -1) {
+    /* get timestamp of the current adapter byte */
+    timestamp = gst_adapter_prev_timestamp (visual->adapter, &dist);
+    if (GST_CLOCK_TIME_IS_VALID (timestamp)) {
+      /* convert bytes to time */
+      dist /= visual->bps;
+      timestamp += gst_util_uint64_scale_int (dist, GST_SECOND, visual->rate);
+    }
+
+    if (timestamp != -1) {
       gint64 qostime;
 
       /* QoS is done on running time */
       qostime = gst_segment_to_running_time (&visual->segment, GST_FORMAT_TIME,
-          visual->next_ts);
+          timestamp);
 
       GST_OBJECT_LOCK (visual);
       /* check for QoS, don't compute buffers that are known to be late */
@@ -685,10 +683,11 @@ gst_visual_chain (GstPad * pad, GstBuffer * buffer)
 
       visual_audio_samplepool_input_channel (visual->audio->samplepool,
           lbuf,
-          rate, VISUAL_AUDIO_SAMPLE_FORMAT_S16, VISUAL_AUDIO_CHANNEL_LEFT);
-      visual_audio_samplepool_input_channel (visual->audio->samplepool,
-          rbuf,
-          rate, VISUAL_AUDIO_SAMPLE_FORMAT_S16, VISUAL_AUDIO_CHANNEL_RIGHT);
+          rate, VISUAL_AUDIO_SAMPLE_FORMAT_S16,
+          (char *) VISUAL_AUDIO_CHANNEL_LEFT);
+      visual_audio_samplepool_input_channel (visual->audio->samplepool, rbuf,
+          rate, VISUAL_AUDIO_SAMPLE_FORMAT_S16,
+          (char *) VISUAL_AUDIO_CHANNEL_RIGHT);
 
       visual_object_unref (VISUAL_OBJECT (lbuf));
       visual_object_unref (VISUAL_OBJECT (rbuf));
@@ -722,17 +721,13 @@ gst_visual_chain (GstPad * pad, GstBuffer * buffer)
     visual_video_set_buffer (visual->video, NULL);
     GST_DEBUG_OBJECT (visual, "rendered one frame");
 
-    GST_BUFFER_TIMESTAMP (outbuf) = visual->next_ts;
+    GST_BUFFER_TIMESTAMP (outbuf) = timestamp;
     GST_BUFFER_DURATION (outbuf) = visual->duration;
 
     ret = gst_pad_push (visual->srcpad, outbuf);
     outbuf = NULL;
 
   skip:
-    /* interpollate next timestamp */
-    if (visual->next_ts != -1)
-      visual->next_ts += visual->duration;
-
     GST_DEBUG_OBJECT (visual, "finished frame, flushing %u samples from input",
         visual->spf);
 
@@ -763,8 +758,8 @@ gst_visual_change_state (GstElement * element, GstStateChange transition)
   switch (transition) {
     case GST_STATE_CHANGE_NULL_TO_READY:
       visual->actor =
-          visual_actor_new (GST_VISUAL_GET_CLASS (visual)->plugin->info->
-          plugname);
+          visual_actor_new (GST_VISUAL_GET_CLASS (visual)->plugin->
+          info->plugname);
       visual->video = visual_video_new ();
       visual->audio = visual_audio_new ();
       /* can't have a play without actors */
@@ -875,6 +870,11 @@ plugin_init (GstPlugin * plugin)
   GST_DEBUG_CATEGORY_INIT (libvisual_debug, "libvisual", 0,
       "libvisual audio visualisations");
 
+#ifdef LIBVISUAL_PLUGINSBASEDIR
+  gst_plugin_add_dependency_simple (plugin, "HOME/.libvisual/actor",
+      LIBVISUAL_PLUGINSBASEDIR "/actor", NULL, GST_PLUGIN_DEPENDENCY_FLAG_NONE);
+#endif
+
   visual_log_set_verboseness (VISUAL_LOG_VERBOSENESS_LOW);
   visual_log_set_info_handler (libvisual_log_handler, (void *) GST_LEVEL_INFO);
   visual_log_set_warning_handler (libvisual_log_handler,
