@@ -1,7 +1,7 @@
 /* GStreamer RIFF I/O
  * Copyright (C) 2003 Ronald Bultje <rbultje@ronald.bitfreak.net>
  *
- * riff.c: plugin registering
+ * riff.c: library initialisation
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -18,6 +18,13 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
+/**
+ * SECTION:gstriff
+ * @short_description: Riff fileformat utillity functions.
+ *
+ * A collection of functions to handle riff base files, such as avi, wav and
+ * asf.
+ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -25,8 +32,15 @@
 
 #include <gst/gst.h>
 
+#include "riff-read.h"
+
 GST_DEBUG_CATEGORY (riff_debug);
 
+/**
+ * gst_riff_init:
+ *
+ * Initialize riff library.
+ */
 void
 gst_riff_init (void)
 {
