@@ -131,17 +131,9 @@ MotifWmHints, MwmHints;
 static void gst_ximagesink_reset (GstXImageSink * ximagesink);
 static void gst_ximagesink_ximage_destroy (GstXImageSink * ximagesink,
     GstXImageBuffer * ximage);
-static void gst_ximagesink_xwindow_update_geometry (GstXImageSink * ximagesink,
-    GstXWindow * xwindow);
+static void gst_ximagesink_xwindow_update_geometry (GstXImageSink * ximagesink);
 static void gst_ximagesink_expose (GstXOverlay * overlay);
 
-/* ElementFactory information */
-static const GstElementDetails gst_ximagesink_details =
-GST_ELEMENT_DETAILS ("Video sink",
-    "Sink/Video",
-    "A standard X based videosink",
-    "Julien Moutte <julien@moutte.net>");
-
 static GstStaticPadTemplate gst_ximagesink_sink_template_factory =
 GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
@@ -719,8 +711,6 @@ gst_ximagesink_ximage_put (GstXImageSink * ximagesink, GstXImageBuffer * ximage)
     }
   }
 
-  gst_ximagesink_xwindow_update_geometry (ximagesink, ximagesink->xwindow);
-
   src.w = ximage->width;
   src.h = ximage->height;
   dst.w = ximagesink->xwindow->width;
@@ -831,8 +821,10 @@ gst_ximagesink_xwindow_set_title (GstXImageSink * ximagesink,
 
       if (title) {
         if ((XStringListToTextProperty (((char **) &title), 1,
-                    &xproperty)) != 0)
+                    &xproperty)) != 0) {
           XSetWMName (ximagesink->xcontext->disp, xwindow->win, &xproperty);
+          XFree (xproperty.value);
+        }
 
         g_free (title_mem);
       }
@@ -925,16 +917,18 @@ gst_ximagesink_xwindow_destroy (GstXImageSink * ximagesink,
 }
 
 static void
-gst_ximagesink_xwindow_update_geometry (GstXImageSink * ximagesink,
-    GstXWindow * xwindow)
+gst_ximagesink_xwindow_update_geometry (GstXImageSink * ximagesink)
 {
   XWindowAttributes attr;
 
-  g_return_if_fail (xwindow != NULL);
   g_return_if_fail (GST_IS_XIMAGESINK (ximagesink));
 
   /* Update the window geometry */
   g_mutex_lock (ximagesink->x_lock);
+  if (G_UNLIKELY (ximagesink->xwindow == NULL)) {
+    g_mutex_unlock (ximagesink->x_lock);
+    return;
+  }
 
   XGetWindowAttributes (ximagesink->xcontext->disp,
       ximagesink->xwindow->win, &attr);
@@ -1077,6 +1071,9 @@ gst_ximagesink_handle_xevents (GstXImageSink * ximagesink)
         exposed = TRUE;
         break;
       case ConfigureNotify:
+        g_mutex_unlock (ximagesink->x_lock);
+        gst_ximagesink_xwindow_update_geometry (ximagesink);
+        g_mutex_lock (ximagesink->x_lock);
         configured = TRUE;
         break;
       default:
@@ -1137,7 +1134,8 @@ gst_ximagesink_event_thread (GstXImageSink * ximagesink)
     if (ximagesink->xwindow) {
       gst_ximagesink_handle_xevents (ximagesink);
     }
-    g_usleep (100000);
+    /* FIXME: do we want to align this with the framerate or anything else? */
+    g_usleep (G_USEC_PER_SEC / 20);
 
     GST_OBJECT_LOCK (ximagesink);
   }
@@ -1146,6 +1144,45 @@ gst_ximagesink_event_thread (GstXImageSink * ximagesink)
   return NULL;
 }
 
+static void
+gst_ximagesink_manage_event_thread (GstXImageSink * ximagesink)
+{
+  GThread *thread = NULL;
+
+  /* don't start the thread too early */
+  if (ximagesink->xcontext == NULL) {
+    return;
+  }
+
+  GST_OBJECT_LOCK (ximagesink);
+  if (ximagesink->handle_expose || ximagesink->handle_events) {
+    if (!ximagesink->event_thread) {
+      /* Setup our event listening thread */
+      GST_DEBUG_OBJECT (ximagesink, "run xevent thread, expose %d, events %d",
+          ximagesink->handle_expose, ximagesink->handle_events);
+      ximagesink->running = TRUE;
+      ximagesink->event_thread = g_thread_create (
+          (GThreadFunc) gst_ximagesink_event_thread, ximagesink, TRUE, NULL);
+    }
+  } else {
+    if (ximagesink->event_thread) {
+      GST_DEBUG_OBJECT (ximagesink, "stop xevent thread, expose %d, events %d",
+          ximagesink->handle_expose, ximagesink->handle_events);
+      ximagesink->running = FALSE;
+      /* grab thread and mark it as NULL */
+      thread = ximagesink->event_thread;
+      ximagesink->event_thread = NULL;
+    }
+  }
+  GST_OBJECT_UNLOCK (ximagesink);
+
+  /* Wait for our event thread to finish */
+  if (thread)
+    g_thread_join (thread);
+
+}
+
+
 /* This function calculates the pixel aspect ratio based on the properties
  * in the xcontext structure and stores it there. */
 static void
@@ -1326,13 +1363,6 @@ gst_ximagesink_xcontext_get (GstXImageSink * ximagesink)
 
   g_mutex_unlock (ximagesink->x_lock);
 
-  /* Setup our event listening thread */
-  GST_OBJECT_LOCK (ximagesink);
-  ximagesink->running = TRUE;
-  ximagesink->event_thread = g_thread_create (
-      (GThreadFunc) gst_ximagesink_event_thread, ximagesink, TRUE, NULL);
-  GST_OBJECT_UNLOCK (ximagesink);
-
   return xcontext;
 }
 
@@ -1562,6 +1592,7 @@ gst_ximagesink_change_state (GstElement * element, GstStateChange transition)
       g_mutex_lock (ximagesink->x_lock);
       XSynchronize (ximagesink->xcontext->disp, ximagesink->synchronous);
       g_mutex_unlock (ximagesink->x_lock);
+      gst_ximagesink_manage_event_thread (ximagesink);
       break;
     case GST_STATE_CHANGE_READY_TO_PAUSED:
       g_mutex_lock (ximagesink->flow_lock);
@@ -1620,13 +1651,13 @@ gst_ximagesink_get_times (GstBaseSink * bsink, GstBuffer * buf,
 }
 
 static GstFlowReturn
-gst_ximagesink_show_frame (GstBaseSink * bsink, GstBuffer * buf)
+gst_ximagesink_show_frame (GstVideoSink * vsink, GstBuffer * buf)
 {
   GstXImageSink *ximagesink;
 
   g_return_val_if_fail (buf != NULL, GST_FLOW_ERROR);
 
-  ximagesink = GST_XIMAGESINK (bsink);
+  ximagesink = GST_XIMAGESINK (vsink);
 
   /* If this buffer has been allocated using our buffer management we simply
      put the ximage which is in the PRIVATE pointer */
@@ -1669,7 +1700,7 @@ gst_ximagesink_show_frame (GstBaseSink * bsink, GstBuffer * buf)
 no_ximage:
   {
     /* No image available. That's very bad ! */
-    GST_DEBUG ("could not create image");
+    GST_WARNING_OBJECT (ximagesink, "could not create image");
     return GST_FLOW_ERROR;
   }
 no_window:
@@ -1769,7 +1800,6 @@ gst_ximagesink_buffer_alloc (GstBaseSink * bsink, guint64 offset, guint size,
   }
 
   /* What is our geometry */
-  gst_ximagesink_xwindow_update_geometry (ximagesink, ximagesink->xwindow);
   dst.w = ximagesink->xwindow->width;
   dst.h = ximagesink->xwindow->height;
 
@@ -1834,6 +1864,7 @@ gst_ximagesink_buffer_alloc (GstBaseSink * bsink, guint64 offset, guint size,
           desired_caps);
       /* we alloc a buffer with the original incomming caps already in the
        * width and height variables */
+      gst_caps_unref (desired_caps);
     }
   }
 
@@ -2025,6 +2056,7 @@ gst_ximagesink_expose (GstXOverlay * overlay)
 {
   GstXImageSink *ximagesink = GST_XIMAGESINK (overlay);
 
+  gst_ximagesink_xwindow_update_geometry (ximagesink);
   gst_ximagesink_ximage_put (ximagesink, NULL);
 }
 
@@ -2128,9 +2160,11 @@ gst_ximagesink_set_property (GObject * object, guint prop_id,
     case PROP_HANDLE_EVENTS:
       gst_ximagesink_set_event_handling (GST_X_OVERLAY (ximagesink),
           g_value_get_boolean (value));
+      gst_ximagesink_manage_event_thread (ximagesink);
       break;
     case PROP_HANDLE_EXPOSE:
       ximagesink->handle_expose = g_value_get_boolean (value);
+      gst_ximagesink_manage_event_thread (ximagesink);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -2281,7 +2315,9 @@ gst_ximagesink_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_set_details (element_class, &gst_ximagesink_details);
+  gst_element_class_set_details_simple (element_class,
+      "Video sink", "Sink/Video",
+      "A standard X based videosink", "Julien Moutte <julien@moutte.net>");
 
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_ximagesink_sink_template_factory));
@@ -2293,10 +2329,12 @@ gst_ximagesink_class_init (GstXImageSinkClass * klass)
   GObjectClass *gobject_class;
   GstElementClass *gstelement_class;
   GstBaseSinkClass *gstbasesink_class;
+  GstVideoSinkClass *videosink_class;
 
   gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
   gstbasesink_class = (GstBaseSinkClass *) klass;
+  videosink_class = (GstVideoSinkClass *) klass;
 
   parent_class = g_type_class_peek_parent (klass);
 
@@ -2337,9 +2375,9 @@ gst_ximagesink_class_init (GstXImageSinkClass * klass)
   gstbasesink_class->buffer_alloc =
       GST_DEBUG_FUNCPTR (gst_ximagesink_buffer_alloc);
   gstbasesink_class->get_times = GST_DEBUG_FUNCPTR (gst_ximagesink_get_times);
-  gstbasesink_class->preroll = GST_DEBUG_FUNCPTR (gst_ximagesink_show_frame);
-  gstbasesink_class->render = GST_DEBUG_FUNCPTR (gst_ximagesink_show_frame);
   gstbasesink_class->event = GST_DEBUG_FUNCPTR (gst_ximagesink_event);
+
+  videosink_class->show_frame = GST_DEBUG_FUNCPTR (gst_ximagesink_show_frame);
 }
 
 /* ============================================================= */
