@@ -58,7 +58,8 @@ static GstPad *mysrcpad, *mysinkpad;
     "width = (int) 240, "               \
     "height = (int) 120, "              \
     "framerate = (fraction) 999/7 , "	\
-    "format = (fourcc) I420"
+    "format = (fourcc) I420, "          \
+    "color-matrix=(string)sdtv"
 
 static GstStaticPadTemplate sinktemplate = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
@@ -78,7 +79,7 @@ static GstStaticPadTemplate srctemplate = GST_STATIC_PAD_TEMPLATE ("src",
     );
 
 static void
-assert_videorate_stats (GstElement * videorate, gchar * reason,
+assert_videorate_stats (GstElement * videorate, const gchar * reason,
     guint64 xin, guint64 xout, guint64 xdropped, guint64 xduplicated)
 {
   guint64 in, out, dropped, duplicated;
@@ -690,7 +691,7 @@ GST_START_TEST (test_upstream_caps_nego)
           GST_STATE_PLAYING) == GST_STATE_CHANGE_SUCCESS,
       "could not set to playing");
 
-  videorate_pad = gst_element_get_pad (videorate, "sink");
+  videorate_pad = gst_element_get_static_pad (videorate, "sink");
   caps = gst_pad_get_caps (videorate_pad);
 
   /* assemble the expected caps */
@@ -721,12 +722,8 @@ GST_START_TEST (test_selected_caps)
   GstBus *bus;
   GstMessage *msg;
 
-  GstPad *videorate_pad;
-  GstCaps *caps = NULL;
-  GstCaps *expected_caps = NULL;
-
-  pipeline = gst_parse_launch ("videotestsrc num-buffers=1 ! "
-      "! identity ! videorate name=videorate0 ! " VIDEO_CAPS_UNUSUAL_FRAMERATE
+  pipeline = gst_parse_launch ("videotestsrc num-buffers=1"
+      " ! identity ! videorate name=videorate0 ! " VIDEO_CAPS_UNUSUAL_FRAMERATE
       " ! fakesink", NULL);
   fail_if (pipeline == NULL);
   videorate = gst_bin_get_by_name (GST_BIN (pipeline), "videorate0");
@@ -741,20 +738,37 @@ GST_START_TEST (test_selected_caps)
       GST_MESSAGE_EOS | GST_MESSAGE_ERROR);
   fail_if (msg == NULL || GST_MESSAGE_TYPE (msg) == GST_MESSAGE_ERROR);
 
-  videorate_pad = gst_element_get_pad (videorate, "sink");
-  g_object_get (videorate_pad, "caps", &caps, NULL);
-  expected_caps = gst_caps_from_string (VIDEO_CAPS_UNUSUAL_FRAMERATE);
+  /* make sure upstream nego works right and videotestsrc has selected the
+   * caps we want downstream of videorate */
+  {
+    GstStructure *s;
+    const GValue *val;
+    GstCaps *caps = NULL;
+    GstPad *videorate_pad;
 
-  fail_unless (gst_caps_is_equal (expected_caps, caps));
+    videorate_pad = gst_element_get_static_pad (videorate, "sink");
+    g_object_get (videorate_pad, "caps", &caps, NULL);
+    fail_unless (caps != NULL);
+
+    GST_DEBUG ("negotiated caps: %" GST_PTR_FORMAT, caps);
+
+    s = gst_caps_get_structure (caps, 0);
+    val = gst_structure_get_value (s, "framerate");
+    fail_unless (val != NULL, "no framerate field in negotiated caps");
+    fail_unless (GST_VALUE_HOLDS_FRACTION (val));
+    fail_unless_equals_int (gst_value_get_fraction_numerator (val), 999);
+    fail_unless_equals_int (gst_value_get_fraction_denominator (val), 7);
+
+    gst_caps_unref (caps);
+    gst_object_unref (videorate_pad);
+  }
 
   /* cleanup */
   gst_object_unref (bus);
   gst_message_unref (msg);
-  gst_caps_unref (caps);
-  gst_caps_unref (expected_caps);
-  gst_object_unref (videorate_pad);
   gst_element_set_state (pipeline, GST_STATE_NULL);
   gst_element_get_state (pipeline, NULL, NULL, GST_CLOCK_TIME_NONE);
+  gst_object_unref (videorate);
   gst_object_unref (pipeline);
 }
 
