@@ -52,8 +52,8 @@ struct paintinfo_struct
 
 struct fourcc_list_struct
 {
-  char *fourcc;
-  char *name;
+  const char *fourcc;
+  const char *name;
   int bitspp;
   void (*paint_setup) (paintinfo * p, unsigned char *dest);
 };
@@ -237,9 +237,9 @@ paint_setup_Y41B (paintinfo * p, unsigned char *dest)
   p->yp = dest;
   p->ystride = GST_ROUND_UP_4 (p->width);
   p->up = p->yp + p->ystride * p->height;
-  p->ustride = GST_ROUND_UP_8 (p->width) / 4;
+  p->ustride = GST_ROUND_UP_16 (p->width) / 4;
   p->vp = p->up + p->ustride * p->height;
-  p->vstride = GST_ROUND_UP_8 (p->width) / 4;
+  p->vstride = GST_ROUND_UP_16 (p->width) / 4;
   p->endptr = p->vp + p->vstride * p->height;
 }
 
@@ -338,6 +338,7 @@ video_format_is_packed (GstVideoFormat fmt)
     case GST_VIDEO_FORMAT_Y42B:
       return FALSE;
     case GST_VIDEO_FORMAT_YUY2:
+    case GST_VIDEO_FORMAT_YVYU:
     case GST_VIDEO_FORMAT_UYVY:
     case GST_VIDEO_FORMAT_AYUV:
     case GST_VIDEO_FORMAT_RGBx:
@@ -393,6 +394,8 @@ GST_START_TEST (test_video_formats)
     for (w = 1; w <= 65; ++w) {
       for (h = 1; h <= 65; ++h) {
         paintinfo paintinfo = { 0, };
+        guint off0, off1, off2, off3;
+        guint size;
 
         GST_LOG ("%s, %dx%d", fourcc_list[i].fourcc, w, h);
 
@@ -409,14 +412,61 @@ GST_START_TEST (test_video_formats)
               paintinfo.vstride);
           /* check component_width * height against offsets/size somehow? */
         }
-        fail_unless_equals_int (gst_video_format_get_component_offset (fmt, 0,
-                w, h), (unsigned long) paintinfo.yp);
-        fail_unless_equals_int (gst_video_format_get_component_offset (fmt, 1,
-                w, h), (unsigned long) paintinfo.up);
-        fail_unless_equals_int (gst_video_format_get_component_offset (fmt, 2,
-                w, h), (unsigned long) paintinfo.vp);
-        fail_unless_equals_int (gst_video_format_get_size (fmt, w, h),
-            (unsigned long) paintinfo.endptr);
+
+        size = gst_video_format_get_size (fmt, w, h);
+        fail_unless_equals_int (size, (unsigned long) paintinfo.endptr);
+
+        off0 = gst_video_format_get_component_offset (fmt, 0, w, h);
+        fail_unless_equals_int (off0, (unsigned long) paintinfo.yp);
+        off1 = gst_video_format_get_component_offset (fmt, 1, w, h);
+        fail_unless_equals_int (off1, (unsigned long) paintinfo.up);
+        off2 = gst_video_format_get_component_offset (fmt, 2, w, h);
+        fail_unless_equals_int (off2, (unsigned long) paintinfo.vp);
+
+        /* should be 0 if there's no alpha component */
+        off3 = gst_video_format_get_component_offset (fmt, 3, w, h);
+        fail_unless_equals_int (off3, (unsigned long) paintinfo.ap);
+
+        /* some gstvideo checks ... (FIXME: fails for Y41B and Y42B; not sure
+         * if the check or the _get_component_size implementation is wrong) */
+        if (fmt != GST_VIDEO_FORMAT_Y41B && fmt != GST_VIDEO_FORMAT_Y42B) {
+          guint cs0, cs1, cs2, cs3;
+
+          cs0 = gst_video_format_get_component_width (fmt, 0, w) *
+              gst_video_format_get_component_height (fmt, 0, h);
+          cs1 = gst_video_format_get_component_width (fmt, 1, w) *
+              gst_video_format_get_component_height (fmt, 1, h);
+          cs2 = gst_video_format_get_component_width (fmt, 2, w) *
+              gst_video_format_get_component_height (fmt, 2, h);
+
+          /* GST_LOG ("cs0=%d,cs1=%d,cs2=%d,off0=%d,off1=%d,off2=%d,size=%d",
+             cs0, cs1, cs2, off0, off1, off2, size); */
+
+          if (!gst_video_format_is_packed (fmt))
+            fail_unless (cs0 <= off1);
+
+          if (gst_video_format_has_alpha (fmt)) {
+            cs3 = gst_video_format_get_component_width (fmt, 3, w) *
+                gst_video_format_get_component_height (fmt, 3, h);
+            fail_unless (cs3 < size);
+            /* U/V/alpha shouldn't take up more space than the Y component */
+            fail_if (cs1 > cs0, "cs1 (%d) should be <= cs0 (%d)", cs1, cs0);
+            fail_if (cs2 > cs0, "cs2 (%d) should be <= cs0 (%d)", cs2, cs0);
+            fail_if (cs3 > cs0, "cs3 (%d) should be <= cs0 (%d)", cs3, cs0);
+
+            /* all components together shouldn't take up more space than size */
+            fail_unless (cs0 + cs1 + cs2 + cs3 <= size);
+          } else {
+            /* U/V shouldn't take up more space than the Y component */
+            fail_if (cs1 > cs0, "cs1 (%d) should be <= cs0 (%d)", cs1, cs0);
+            fail_if (cs2 > cs0, "cs2 (%d) should be <= cs0 (%d)", cs2, cs0);
+
+            /* all components together shouldn't take up more space than size */
+            fail_unless (cs0 + cs1 + cs2 <= size,
+                "cs0 (%d) + cs1 (%d) + cs2 (%d) should be <= size (%d)",
+                cs0, cs1, cs2, size);
+          }
+        }
       }
     }
   }
@@ -480,8 +530,8 @@ GST_START_TEST (test_parse_caps_rgb)
     int w = -1, h = -1;
 
     caps = gst_caps_from_string (formats[i].tmpl_caps_string);
-    gst_caps_set_simple (caps, "width", G_TYPE_INT, 2 * i, "height",
-        G_TYPE_INT, i, "framerate", GST_TYPE_FRACTION, 15, 1,
+    gst_caps_set_simple (caps, "width", G_TYPE_INT, 2 * (i + 1), "height",
+        G_TYPE_INT, i + 1, "framerate", GST_TYPE_FRACTION, 15, 1,
         "pixel-aspect-ratio", GST_TYPE_FRACTION, 1, 1, NULL);
     g_assert (gst_caps_is_fixed (caps));
 
@@ -489,8 +539,8 @@ GST_START_TEST (test_parse_caps_rgb)
 
     fail_unless (gst_video_format_parse_caps (caps, &fmt, &w, &h));
     fail_unless_equals_int (fmt, formats[i].fmt);
-    fail_unless_equals_int (w, 2 * i);
-    fail_unless_equals_int (h, i);
+    fail_unless_equals_int (w, 2 * (i + 1));
+    fail_unless_equals_int (h, i + 1);
 
     /* make sure they're serialised back correctly */
     caps2 = gst_video_format_new_caps (fmt, w, h, 15, 1, 1, 1);
@@ -504,6 +554,32 @@ GST_START_TEST (test_parse_caps_rgb)
 
 GST_END_TEST;
 
+GST_START_TEST (test_events)
+{
+  GstEvent *e;
+  gboolean in_still;
+
+  e = gst_video_event_new_still_frame (TRUE);
+  fail_if (e == NULL, "Failed to create still frame event");
+  fail_unless (gst_video_event_parse_still_frame (e, &in_still),
+      "Failed to parse still frame event");
+  fail_unless (gst_video_event_parse_still_frame (e, NULL),
+      "Failed to parse still frame event w/ in_still == NULL");
+  fail_unless (in_still == TRUE);
+  gst_event_unref (e);
+
+  e = gst_video_event_new_still_frame (FALSE);
+  fail_if (e == NULL, "Failed to create still frame event");
+  fail_unless (gst_video_event_parse_still_frame (e, &in_still),
+      "Failed to parse still frame event");
+  fail_unless (gst_video_event_parse_still_frame (e, NULL),
+      "Failed to parse still frame event w/ in_still == NULL");
+  fail_unless (in_still == FALSE);
+  gst_event_unref (e);
+}
+
+GST_END_TEST;
+
 static Suite *
 video_suite (void)
 {
@@ -514,6 +590,7 @@ video_suite (void)
   tcase_add_test (tc_chain, test_video_formats);
   tcase_add_test (tc_chain, test_dar_calc);
   tcase_add_test (tc_chain, test_parse_caps_rgb);
+  tcase_add_test (tc_chain, test_events);
 
   return s;
 }
